/**
 * Copyright 2012 Bryan Miller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package lib.obsidianblk.midnightgdx;


public final class Attributes {
	public static final int VERTEX = 0x0001;
	public static final int COLOR = 0x0010;
	public static final int NORMAL = 0x0100;
	public static final int TEXCOORD = 0x1000;
	public static final int UNDEFINED = 0x0000;
	public static final int ALL = Attributes.VERTEX&Attributes.COLOR&Attributes.NORMAL&Attributes.TEXCOORD;
	
	private int AID = 0x0000;
	
	public static boolean OneOf(int a){
		return a == Attributes.VERTEX || a == Attributes.COLOR || a == Attributes.NORMAL || a == Attributes.TEXCOORD;
	}
	
	public static boolean ValidAttribValue(int a, boolean MustContainVertex){
		if (MustContainVertex){
			if ((a&Attributes.VERTEX) == 0){
				return false;
			}
		}
		return a > Attributes.UNDEFINED && a <= Attributes.ALL;
	}
	
	public Attributes(int attrib){
		Support(attrib);
	}
	
	public Attributes(){
		Support(Attributes.VERTEX);
	}
	
	public boolean Supports(int attrib, boolean exactMatch){
		if (attrib > Attributes.UNDEFINED && attrib <= Attributes.ALL){
			if (exactMatch){
				return AID == attrib;
			}
			return (AID & attrib) != 0;
		}
		return false;
	}
	public boolean Supports(int attrib){
		return Supports(attrib, true);
	}
	
	public void Support(int attrib){
		if (attrib > Attributes.UNDEFINED && attrib <= Attributes.ALL){
			AID |= attrib;
		}
	}
	
	public int asInt(){
		return AID;
	}

	
	@Override
	public boolean equals(Object obj) {
		boolean Result = false;
		if (obj instanceof Attributes){
			Attributes that = (Attributes)obj;
			Result = this.Supports(that.asInt());
		}
		return Result;
	}

	@Override
	protected Object clone() throws CloneNotSupportedException {
		return new Attributes(AID);
	}

	@Override
	public String toString() {
		boolean ListStarted = false;
		String a = "Attributes (";
		if (AID != Attributes.UNDEFINED){
			if (Supports(Attributes.VERTEX)){
				a.concat("VERTEX");
				ListStarted = true;
			}
			if (Supports(Attributes.COLOR)){
				if (ListStarted){
					a.concat(", ");
				}
				a.concat("COLOR");
				ListStarted = true;
			}
			if (Supports(Attributes.NORMAL)){
				if (ListStarted){
					a.concat(", ");
				}
				a.concat("NORMAL");
				ListStarted = true;
			}
			if (Supports(Attributes.TEXCOORD)){
				if (ListStarted){
					a.concat(", ");
				}
				a.concat("TEXCOORD");
				ListStarted = true;
			}
		} else {
			a.concat("UNDEFINED");
		}
		a.concat(")");
		
		return a;
	}
	
	
}


