/**
 * Copyright 2012 Bryan Miller
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



package lib.obsidianblk.midnightgdx;

import java.nio.IntBuffer;

import lib.obsidianblk.midnightgdx.exceptions.ShaderCompilerException;
import lib.obsidianblk.midnightgdx.exceptions.ShaderDefException;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.GL20;


public class Shader {
	private String Definition = null;
	private int SID = 0;
	private int GLShaderType = -1;
	
	private IntBuffer compileStatus = IntBuffer.allocate(1);
	
	public Shader(String Def, int GL_SHADER_TYPE) throws ShaderDefException{
		if (GL_SHADER_TYPE != GL20.GL_VERTEX_SHADER && GL_SHADER_TYPE != GL20.GL_FRAGMENT_SHADER){
			throw new ShaderDefException("Unknown shader type.");
		}
		Definition = Def;
		GLShaderType = GL_SHADER_TYPE;
	}
	
	public int GetShaderGLID(){
		return SID;
	}
	
	public int GetShaderType(){
		return GLShaderType;
	}
	
	public int Compile() throws ShaderCompilerException{
		if (SID <= 0){
			
			SID = Gdx.gl20.glCreateShader(GLShaderType);
			Gdx.gl20.glShaderSource(SID, Definition);
			Gdx.gl20.glCompileShader(SID);
			
			Gdx.gl20.glGetShaderiv(SID, GL20.GL_COMPILE_STATUS, compileStatus);
			if (compileStatus.get(0) == 0){
				Gdx.gl20.glDeleteShader(SID);
				SID = 0;
				if (GLShaderType == GL20.GL_VERTEX_SHADER){
					throw new ShaderCompilerException("Error creating vertex shader.");
				} else if (GLShaderType == GL20.GL_FRAGMENT_SHADER){
					throw new ShaderCompilerException("Error creating fragment shader.");
				}
			}
		}
		
		return SID;
	}
	
	public void Decompile(){
		if (SID > 0){
			Gdx.gl20.glDeleteShader(SID);
			SID = 0;
		}
	}
	
	public boolean IsCompiled(){return (SID>0);}
}




